unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, AVRootIntf, StdCtrls;

type
  TForm1 = class(TForm, IApplication)
    Memo1: TMemo;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    CBPort: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure Button4Click(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { Private-Deklarationen }
    FLoader: IAVRootloader;
    FMode: TMode;
    FCompile: Boolean;
  protected
    { implements IApplication Interface}
    function ProcessMessages: Bool; stdcall;
    procedure Changed; stdcall;
    procedure Output(const Msg: WideString; Code: Integer); stdcall;

    function GetFLASHFileName: WideString; stdcall;
    function GetEEPROMFileName: WideString; stdcall;
    function GetACYFileName: WideString; stdcall;
    function GetPassword: WideString; stdcall;
    function GetBootSign: WideString; stdcall;
    function GetTimeouts: TTimeouts; stdcall;
    function GetAppCmd: WideString; stdcall;
    function GetAppCmdResponse: WideString; stdcall;
    function GetAppVersion(Masked: Bool = False): Integer; stdcall;
    function GetACYInfo: WideString; stdcall;

    function OpenCommunication(Index: Integer): ICOM; stdcall;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

  Port: String = 'COM4';
  Baudrate: Cardinal = 115200;

implementation

{$R *.dfm}

function TForm1.ProcessMessages: Bool;
begin
  result := false;
end;

procedure TForm1.Changed;
const
  sYesNo: array[Boolean] of String = ('No', 'Yes');
  sWire: array[Boolean] of String = ('2-Wire', '1-Wire');
begin
  if FMode <> FLoader.Mode then
  begin
    if not (FMode in [moConnected, moWorking]) and (FLoader.Mode in [moConnected, moWorking]) then
      with FLoader.Device do
      begin
        Memo1.Clear;
        Memo1.Lines.Add(Format('Connection                     : %6s', [sWire[FLoader.COM.EchoMode]]));
        Memo1.Lines.Add(Format('Device name                    : %s', [Name]));
        Memo1.Lines.Add(Format('Device signature               : %0.6x', [Signature]));
        Memo1.Lines.Add(Format('SRAM size                      : %6d Byte', [RamSize]));
        Memo1.Lines.Add(Format('EEPROM size                    : %6d Byte', [EepromSize]));
        Memo1.Lines.Add(Format('FLASH size                     : %6d Byte', [FlashSize]));
        Memo1.Lines.Add(Format('FLASH size for application     : %6d Byte', [AppFlashSize]));
        Memo1.Lines.Add(Format('FLASH pagesize                 : %6d Byte', [PageSize]));
        Memo1.Lines.Add(Format('Bootloader size                : %6d Byte', [FlashSize - AppFlashSize]));
        Memo1.Lines.Add(Format('Buffersize for data            : %6d Byte', [RamSize - PageSize + 24]));
        Memo1.Lines.Add(Format('SRAM start address             : %6d', [RamStartAddress]));
        Memo1.Lines.Add(Format('Bootloader version             : %6d', [Version]));
        Memo1.Lines.Add(Format('Use bootsection                : %6s', [sYesNo[UseBootSection]]));
        Memo1.Lines.Add(Format('Cryptography supported         : %6s', [sYesNo[Support and sfCrypt <> 0]]));
        if Support and sfCrypt <> 0 then
        begin
          Memo1.Lines.Add(Format('FLASH data must be encrypted   : %6s', [sYesNo[Support and sfCryptFlash <> 0]]));
          Memo1.Lines.Add(Format('EEPROM data must be encrypted  : %6s', [sYesNo[Support and sfCryptEeprom <> 0]]));
        end;
        if Info <> '' then
          Memo1.Lines.Add(Format('Info                           : %s', [Info]));
      end;

    FMode := FLoader.Mode;

    Button1.Enabled := FMode in [moDisconnected, moConnected];
    Button2.Enabled := FMode in [moDisconnected, moConnected];
    Button3.Enabled := FMode in [moDisconnected, moConnected];
    Button4.Enabled := FMode <> moWorking;
    case FMode of
      moDisconnected: Button4.Caption := 'Connect';
      moConnected: Button4.Caption := 'Disconnect';
      moConnecting: Button4.Caption := 'Abort connecting...';
      moWorking: Button4.Caption := 'Working..';
    end;
  end;
end;

procedure TForm1.Output(const Msg: WideString; Code: Integer);
begin
  Memo1.Lines.Add(Msg);
end;

function TForm1.GetFLASHFileName: WideString;
begin
  Result := '';
end;

function TForm1.GetEEPROMFileName: WideString;
begin
  Result := '';
end;

function TForm1.GetACYFileName: WideString;
begin
  Result := '';
end;

function TForm1.GetPassword: WideString;
begin
  Result := InputBox('Password required', 'Password', '');
end;

function TForm1.GetBootSign: WideString;
begin
  Result := 'Bootloader';
end;

function TForm1.GetTimeouts: TTimeouts;
begin
  Result.Connect := 100;
  Result.Base := 100;
  Result.Erase := 10;
  Result.Flash := 50;
  Result.Eeprom := 10;
  Result.Buffer := 1;
  Result.AppCmd := 20;
  Result.KeepAlive := 100;
  Result.RTSPulse := 0;
  Result.RTSInterval := 0;
  Result.ConnectTrials := 25;
  Result.MaxPacketSize := 0;
  Result.Options := 1;
end;

function TForm1.GetAppCmd: WideString;
begin
   Result := chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF)+chr($FF);
end;

function TForm1.GetAppCmdResponse: WideString;
begin
  Result := '';
end;

function TForm1.GetAppVersion(Masked: Bool = False): Integer;
begin
  Result := $01000000;
end;

function TForm1.GetACYInfo: WideString;
begin
  Result := '';
end;

function TForm1.OpenCommunication(Index: Integer): ICOM;
var
  PortName: String;
begin
  Result := nil;
  if AnsiCompareText(Port, 'AUTO') = 0 then
  begin
    if Index >= CBPort.Items.Count then
    begin
      Output('Error: no Device found', ocError);
      Exit;
    end;
    PortName := CBPort.Items[Index];
    try
      Result := OpenCOM('\\.\' + PortName, Self);
    except
      Output(Format('Info: can not open port %s', [PortName]), ocInfo);
      raise EAbort.Create('');
    end;
  end else
  begin
    if Index > 1 then Exit;
    PortName := Port;
    try
      Result := OpenCOM('\\.\' + PortName, Self);
    except
      raise Exception.CreateFmt('Error: can not open port %s', [PortName]);
    end;
  end;
  try
    Result.SetParams(Baudrate);
  except
    raise Exception.CreateFmt('Error: can not setup baudrate to %d', [Baudrate]);
  end;
  Output('Connecting on port ' + PortName + '...', ocInfo);
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FLoader := AVRootIntf.OpenAVRootloader(Self);
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := FLoader.Mode = moDisconnected;
  if not CanClose then
    FLoader.DoDisconnect;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  FCompile := True;
  try
    FLoader.DoCompile(True, True);
  finally
    FCompile := False;
  end;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  FLoader.DoProgram(True, True);
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  FLoader.DoEraseFlash;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  case FLoader.Mode of
    moDisconnected: FLoader.DoConnect(false);
    moConnected: FLoader.DoDisconnect;
  else
    FLoader.DoAbort;
  end;
end;

end.
